/******************************************************************************************/
/****************************Includieren der Headerdateien*********************************/
/******************************************************************************************/

#include <SHT_11.h>     //Anschlussdefinition des SHT11 + Steuerbefehle          

/******************************************************************************************/
/***************************Transmissionstart Sequenz an SHT11*****************************/
/******************************************************************************************/

void SHT_Transstart (void)
{
MAKE_DATA_OUTPUT;       //Datenpin des SHT11 als Ausgang
MAKE_SCK_OUTPUT;        //Takt des SHT11 als Ausgang
_delay_us(EINPEGELN);
SET_DATA_PIN;           //Datenpin auf High
_delay_us(EINPEGELN);
RESET_SCK_PIN;          //Taktpin auf Low       
_delay_us(EINPEGELN);
SET_SCK_PIN;            //Taktpin auf High
_delay_us(EINPEGELN);
RESET_DATA_PIN;         //Datenpin auf Low
_delay_us(EINPEGELN);
RESET_SCK_PIN;          //Taktpin auf Low
_delay_us(EINPEGELN);
SET_SCK_PIN;            //Taktpin auf High
_delay_us(EINPEGELN);
SET_DATA_PIN;           //Datenpin auf High
_delay_us(EINPEGELN);
RESET_SCK_PIN;          //Taktpin auf Low
_delay_us(EINPEGELN);
}

/******************************************************************************************/
/********************************Connectionreset an SHT11**********************************/
/******************************************************************************************/

void SHT_Connectionreset (void)
{
unsigned char i = 0x00; //Variable fr Zhlschleife

MAKE_DATA_OUTPUT;       //Datenpin des SHT11 als Ausgang
MAKE_SCK_OUTPUT;        //Taktpin des SHT11 als Ausgang
_delay_us(EINPEGELN);
SET_DATA_PIN;           //Datenpin auf High
RESET_SCK_PIN;          //Taktpin auf Low
for(i=0;i<9;i++)        //Schleife wird 9x ausgefhrt
 { 
  SET_SCK_PIN;          //Taktpin auf High     
  _delay_us(EINPEGELN);
  RESET_SCK_PIN;        //Taktpin auf Low
  _delay_us(EINPEGELN);
 }
SHT_Transstart();       //Ausgeben der nachfolgenden Transmissionstart Sequenz
}

/******************************************************************************************/
/**************************Byte an SHT11 senden und ACK berprfen*************************/
/******************************************************************************************/

char SHT_Write_byte (unsigned char value)
{ 
unsigned char i     = 0x00;     
unsigned char error = 0x00;     

MAKE_SCK_OUTPUT;                

for (i=0x80;i>0;i/=2)           
{ 
   if (i & value)               
    {                           
     SET_DATA_PIN;              
     _delay_us(EINPEGELN);
    }          
   else                         
    {
     RESET_DATA_PIN;           
     _delay_us(EINPEGELN); 
    }    
_delay_us(EINPEGELN);                                
SET_SCK_PIN;                    //Taktpin auf High (CLK)
_delay_us(EINPEGELN);
RESET_SCK_PIN;                  //Taktpin auf Low
_delay_us(EINPEGELN);
}

_delay_us(EINPEGELN);
SET_DATA_PIN;                   //Datenpin auf High            
_delay_us(EINPEGELN);
SET_SCK_PIN;                    //Taktpin auf High (ACK)        
_delay_us(EINPEGELN);
MAKE_DATA_INPUT;                //Datenpin des SHT11 als Eingang

if ((SHT_DATA_PIN_IN & (1<<SHT_DATA_PIN)))  //Wenn Datenpin (ACK) = 1 dann:
{
 error = 1;                     //setze "error" auf 1
}
else                            //ist ACK = Low (kein Fehler)
{
 error = 0;                     //setze "error" auf 0
}

_delay_us(EINPEGELN);
RESET_SCK_PIN;                  //Setze den Taktbin auf Low

_delay_ms(400);                 //Warte auf das Wandlungsergebniss
 
return (error);                 //"error" = 1, wenn Fehler aufgetreten (sonst 0)
}

/******************************************************************************************/
/**************Byte aus SHT11 lesen, ACK prfen und ACK ausgeben (bei ACK = 1)*************/
/******************************************************************************************/

char SHT_Read_byte(unsigned char ack)
{ 
unsigned char i   = 0x00;       
unsigned char val = 0x00;      

MAKE_DATA_OUTPUT;               //Datenpin des SHT11 als Ausgang
_delay_us(EINPEGELN);
SET_DATA_PIN;                   //Datenpin auf Ausgang
_delay_us(EINPEGELN);
MAKE_DATA_INPUT;                //Datenpin des SHT11 als Eingang
                     
for (i=0x80;i>0;i/=2)           //Verschiebe Bits fr Maskierung
{   
 SET_SCK_PIN;                   //Taktpin auf High (CLK)
 _delay_us(EINPEGELN);                
 if ((SHT_DATA_PIN_IN & (1<<SHT_DATA_PIN)))     //Wenn Datenpin = 1, dann:
  {
   val=(val | i);               //schreibe Wert in "val"
  }       
 RESET_SCK_PIN;                 //Taktpin auf Low
 _delay_us(EINPEGELN);
}

_delay_us(EINPEGELN);
MAKE_DATA_OUTPUT;               //Datenpin des SHT11 als Ausgang
_delay_us(EINPEGELN);

if(!ack == 1)                   //Wenn fr ACK eine 1 bergeben wurde, dann:
{
 SET_DATA_PIN;                  //Datenpin auf High
}
else
{
 RESET_DATA_PIN                 //Datenpin auf Low
}

_delay_us(EINPEGELN);
SET_SCK_PIN;                    //Taktpin auf High (ACK)
_delay_us(EINPEGELN); 
RESET_SCK_PIN;                  //Taktpin auf Low
_delay_us(EINPEGELN);
SET_DATA_PIN;                   //Datenpin auf High
_delay_us(EINPEGELN);

return (val);
}

/******************************************************************************************/
/***********************Auslesen und Rckgeben (Float) der Temperatur**********************/
/******************************************************************************************/

float SHT_Temperatur (void)
{
uint8_t  TEMP_MSB       = 0x00;         //Variable fr Temperatur MSN (erstes Byte)    
uint8_t  TEMP_LSB       = 0x00;         //Variable fr Temperatur LSB (zweites Byte) 
uint16_t TEMP_16B       = 0x00;         //Variable fr 16 Bit Temperaturwert
float    TEMP_FLOAT     = 0x00;         //Variable fr Float Temperaturwert

SHT_Connectionreset();                  //Ausfhren eines Connectionresets
SHT_Transstart();                       //Ausgeben der Transmissionstart Sequenz
SHT_Write_byte(0x03);                   //Schreibe Byte fr Temp. (Adr.000, Com. 00011)

TEMP_MSB = SHT_Read_byte(0x01);         //Lese das Temperatur MSN aus (mit ACK -> nchstes Byte)
TEMP_LSB = SHT_Read_byte(0x00);         //Lese das Temperatur LSB aus (ohne ACK -> ohne Checksumme)

TEMP_16B = TEMP_MSB << 8;               //Fge das Temp. MSB ein und verschiebe es um 8 Stellen
TEMP_16B = TEMP_16B | TEMP_LSB;         //Lege das Temp. LSB drber, sodass ein 16 Bit Wert entsteht

TEMP_FLOAT = TEMP_16B;                  //schreibe den "uint16_t" Wert in ein "float" Wert
TEMP_FLOAT = TEMP_FLOAT * 0.01 - 40;    //Umrechnung des Wertes in eine Temperatur (nach Datenblatt)


return(TEMP_FLOAT);                     //Rckgabe des Temperaturwertes ("float")
}

/******************************************************************************************/
/*************************Auslesen und Rckgeben (Float) der Feuchte***********************/
/******************************************************************************************/

float SHT_Feuchte (void)
{
uint8_t  FEUCHTE_MSB         = 0x00;    //Variable fr Feuchte MSN (erstes Byte) 
uint8_t  FEUCHTE_LSB         = 0x00;    //Variable fr Feuchte MSN (erstes Byte) 
uint16_t FEUCHTE_16B         = 0x00;    //Variable fr 16 Bit Feuchtewert
float    FEUCHTE_FLOAT       = 0x00;    //Variable fr Float Feuchtewert
float    FEUCHTE_RECHNUNG_1  = 0x00;    //Variable fr Float Umrechnung (temporr)
float    FEUCHTE_RECHNUNG_2  = 0x00;    //Variable fr Float Umrechnung (temporr)

SHT_Connectionreset();                  //Ausfhren eines Connectionresets
SHT_Transstart();                       //Ausgeben der Transmissionstart Sequenz
SHT_Write_byte(0x05);                   //Schreibe Byte fr Feuchte (Adr.000, Com. 00101)

FEUCHTE_MSB = SHT_Read_byte(0x01);      //Lese das Feuchte MSN aus (mit ACK -> nchstes Byte)
FEUCHTE_LSB = SHT_Read_byte(0x00);      //Lese das Temperatur LSB aus (ohne ACK -> ohne Checksumme)

FEUCHTE_16B = FEUCHTE_MSB << 8;         //Fge das Feuchte MSB ein und verschiebe es um 8 Stellen
FEUCHTE_16B = FEUCHTE_16B | FEUCHTE_LSB;//Lege das Feuchte. LSB drber, sodass ein 16 Bit Wert entsteht

FEUCHTE_FLOAT      = FEUCHTE_16B;       //schreibe den "uint16_t" Wert in ein "float" Wert
FEUCHTE_RECHNUNG_1 = FEUCHTE_FLOAT;     //schreibe Wert in eine zweite VAR fr Rechnung

//Umrechnung des Wertes in eine Feuchtigkeit (nach Datenblatt)
FEUCHTE_RECHNUNG_1 = -0.0000028 * FEUCHTE_FLOAT * FEUCHTE_FLOAT + 0.0405 * FEUCHTE_FLOAT + -4.0;
FEUCHTE_RECHNUNG_2 = FEUCHTE_RECHNUNG_1;
FEUCHTE_RECHNUNG_1 = (SHT_Temperatur()-25) * (0.01 + 0.00008 * FEUCHTE_FLOAT) + FEUCHTE_RECHNUNG_2;

FEUCHTE_FLOAT    = FEUCHTE_RECHNUNG_1;  //Ergebniss in Variable schreiben

return(FEUCHTE_FLOAT);                  //Rckgabe des Feuchewertes ("float")
}

/******************************************************************************************/
/****************************************Programmende**************************************/
/******************************************************************************************/
